@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
-
 # Copyright (C) 2006  Joe Wreschnig
 #
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License version 2 as
-# published by the Free Software Foundation.
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 
 """Read and write MPEG-4 audio files with iTunes metadata.
 
@@ -28,8 +28,8 @@ import sys
 
 from mutagen import FileType, Tags, StreamInfo, PaddingInfo
 from mutagen._constants import GENRES
-from mutagen._util import (cdata, insert_bytes, DictProxy, MutagenError,
-                           hashable, enum, get_size, resize_bytes)
+from mutagen._util import cdata, insert_bytes, DictProxy, MutagenError, \
+    hashable, enum, get_size, resize_bytes, loadfile, convert_error
 from mutagen._compat import (reraise, PY2, string_types, text_type, chr_,
                              iteritems, PY3, cBytesIO, izip, xrange)
 from ._atom import Atoms, Atom, AtomError
@@ -37,7 +37,7 @@ from ._util import parse_full_atom
 from ._as_entry import AudioSampleEntry, ASEntryError
 
 
-class error(IOError, MutagenError):
+class error(MutagenError):
     pass
 
 
@@ -49,6 +49,10 @@ class MP4StreamInfoError(error):
     pass
 
 
+class MP4NoTrackError(MP4StreamInfoError):
+    pass
+
+
 class MP4MetadataValueError(ValueError, MP4MetadataError):
     pass
 
@@ -58,7 +62,7 @@ __all__ = ['MP4', 'Open', 'delete', 'MP4Cover', 'MP4FreeForm', 'AtomDataType']
 
 @enum
 class AtomDataType(object):
-    """Enum for `dataformat` attribute of MP4FreeForm.
+    """Enum for ``dataformat`` attribute of MP4FreeForm.
 
     .. versionadded:: 1.25
     """
@@ -132,8 +136,8 @@ class MP4Cover(bytes):
     """A cover artwork.
 
     Attributes:
-
-    * imageformat -- format of the image (either FORMAT_JPEG or FORMAT_PNG)
+        imageformat (`AtomDataType`): format of the image
+            (either FORMAT_JPEG or FORMAT_PNG)
     """
 
     FORMAT_JPEG = AtomDataType.JPEG
@@ -168,8 +172,7 @@ class MP4FreeForm(bytes):
     """A freeform value.
 
     Attributes:
-
-    * dataformat -- format of the data (see AtomDataType)
+        dataformat (`AtomDataType`): format of the data (see AtomDataType)
     """
 
     FORMAT_DATA = AtomDataType.IMPLICIT  # deprecated
@@ -201,7 +204,6 @@ class MP4FreeForm(bytes):
             AtomDataType(self.dataformat))
 
 
-
 def _name2key(name):
     if PY2:
         return name
@@ -253,7 +255,9 @@ def _item_sort_key(key, value):
 
 
 class MP4Tags(DictProxy, Tags):
-    r"""Dictionary containing Apple iTunes metadata list key/values.
+    r"""MP4Tags()
+
+    Dictionary containing Apple iTunes metadata list key/values.
 
     Keys are four byte identifiers, except for freeform ('----')
     keys. Values are usually unicode strings, but some atoms have a
@@ -286,6 +290,8 @@ class MP4Tags(DictProxy, Tags):
     * 'soco' -- composer sort order
     * 'sosn' -- show sort order
     * 'tvsh' -- show name
+    * '\\xa9wrk' -- work
+    * '\\xa9mvn' -- movement
 
     Boolean values:
 
@@ -298,9 +304,21 @@ class MP4Tags(DictProxy, Tags):
     * 'trkn' -- track number, total tracks
     * 'disk' -- disc number, total discs
 
+    Integer values:
+
+    * 'tmpo' -- tempo/BPM
+    * '\\xa9mvc' -- Movement Count
+    * '\\xa9mvi' -- Movement Index
+    * 'shwm' -- work/movement
+    * 'stik' -- Media Kind
+    * 'rtng' -- Content Rating
+    * 'tves' -- TV Episode
+    * 'tvsn' -- TV Season
+    * 'plID', 'cnID', 'geID', 'atID', 'sfID', 'cmID', 'akID' -- Various iTunes
+      Internal IDs
+
     Others:
 
-    * 'tmpo' -- tempo/BPM, 16 bit int
     * 'covr' -- cover artwork, list of MP4Cover objects (which are
       tagged strs)
     * 'gnre' -- ID3v1 genre. Not supported, use '\\xa9gen' instead.
@@ -365,13 +383,16 @@ class MP4Tags(DictProxy, Tags):
         atom_name = _key2name(key)[:4]
         if atom_name in self.__atoms:
             render_func = self.__atoms[atom_name][1]
+            render_args = self.__atoms[atom_name][2:]
         else:
             render_func = type(self).__render_text
+            render_args = []
 
-        return render_func(self, key, value)
+        return render_func(self, key, value, *render_args)
 
-    def save(self, filename, padding=None):
-        """Save the metadata to the given filename."""
+    @convert_error(IOError, error)
+    @loadfile(writable=True)
+    def save(self, filething, padding=None):
 
         values = []
         items = sorted(self.items(), key=lambda kv: _item_sort_key(*kv))
@@ -395,13 +416,12 @@ class MP4Tags(DictProxy, Tags):
         data = Atom.render(b"ilst", b"".join(values))
 
         # Find the old atoms.
-        with open(filename, "rb+") as fileobj:
-            try:
-                atoms = Atoms(fileobj)
-            except AtomError as err:
-                reraise(error, err, sys.exc_info()[2])
+        try:
+            atoms = Atoms(filething.fileobj)
+        except AtomError as err:
+            reraise(error, err, sys.exc_info()[2])
 
-            self.__save(fileobj, atoms, data, padding)
+        self.__save(filething.fileobj, atoms, data, padding)
 
     def __save(self, fileobj, atoms, data, padding):
         try:
@@ -660,30 +680,59 @@ class MP4Tags(DictProxy, Tags):
         key = _name2key(b"\xa9gen")
         self.__add(key, values)
 
-    def __parse_tempo(self, atom, data):
+    def __parse_integer(self, atom, data):
         values = []
         for version, flags, data in self.__parse_data(atom, data):
-            # version = 0, flags = 0 or 21
-            if len(data) != 2:
-                raise MP4MetadataValueError("invalid tempo")
-            values.append(cdata.ushort_be(data))
+            if version != 0:
+                raise MP4MetadataValueError("unsupported version")
+            if flags not in (AtomDataType.IMPLICIT, AtomDataType.INTEGER):
+                raise MP4MetadataValueError("unsupported type")
+
+            if len(data) == 1:
+                value = cdata.int8(data)
+            elif len(data) == 2:
+                value = cdata.int16_be(data)
+            elif len(data) == 3:
+                value = cdata.int32_be(data + b"\x00") >> 8
+            elif len(data) == 4:
+                value = cdata.int32_be(data)
+            elif len(data) == 8:
+                value = cdata.int64_be(data)
+            else:
+                raise MP4MetadataValueError(
+                    "invalid value size %d" % len(data))
+            values.append(value)
+
         key = _name2key(atom.name)
         self.__add(key, values)
 
-    def __render_tempo(self, key, value):
+    def __render_integer(self, key, value, min_bytes):
+        assert min_bytes in (1, 2, 4, 8)
+
+        data_list = []
         try:
-            if len(value) == 0:
-                return self.__render_data(key, 0, AtomDataType.INTEGER, b"")
+            for v in value:
+                # We default to the int size of the usual values written
+                # by itunes for compatibility.
+                if cdata.int8_min <= v <= cdata.int8_max and min_bytes <= 1:
+                    data = cdata.to_int8(v)
+                if cdata.int16_min <= v <= cdata.int16_max and min_bytes <= 2:
+                    data = cdata.to_int16_be(v)
+                elif cdata.int32_min <= v <= cdata.int32_max and \
+                        min_bytes <= 4:
+                    data = cdata.to_int32_be(v)
+                elif cdata.int64_min <= v <= cdata.int64_max and \
+                        min_bytes <= 8:
+                    data = cdata.to_int64_be(v)
+                else:
+                    raise MP4MetadataValueError(
+                        "value out of range: %r" % value)
+                data_list.append(data)
 
-            if (min(value) < 0) or (max(value) >= 2 ** 16):
-                raise MP4MetadataValueError(
-                    "invalid 16 bit integers: %r" % value)
-        except TypeError:
-            raise MP4MetadataValueError(
-                "tmpo must be a list of 16 bit integers")
+        except (TypeError, ValueError, cdata.error) as e:
+            raise MP4MetadataValueError(e)
 
-        values = [cdata.to_ushort_be(v) for v in value]
-        return self.__render_data(key, 0, AtomDataType.INTEGER, values)
+        return self.__render_data(key, 0, AtomDataType.INTEGER, data_list)
 
     def __parse_bool(self, atom, data):
         for version, flags, data in self.__parse_data(atom, data):
@@ -785,10 +834,24 @@ class MP4Tags(DictProxy, Tags):
         b"trkn": (__parse_pair, __render_pair),
         b"disk": (__parse_pair, __render_pair_no_trailing),
         b"gnre": (__parse_genre, None),
-        b"tmpo": (__parse_tempo, __render_tempo),
+        b"plID": (__parse_integer, __render_integer, 8),
+        b"cnID": (__parse_integer, __render_integer, 4),
+        b"geID": (__parse_integer, __render_integer, 4),
+        b"atID": (__parse_integer, __render_integer, 4),
+        b"sfID": (__parse_integer, __render_integer, 4),
+        b"cmID": (__parse_integer, __render_integer, 4),
+        b"akID": (__parse_integer, __render_integer, 1),
+        b"tvsn": (__parse_integer, __render_integer, 4),
+        b"tves": (__parse_integer, __render_integer, 4),
+        b"tmpo": (__parse_integer, __render_integer, 2),
+        b"\xa9mvi": (__parse_integer, __render_integer, 2),
+        b"\xa9mvc": (__parse_integer, __render_integer, 2),
         b"cpil": (__parse_bool, __render_bool),
         b"pgap": (__parse_bool, __render_bool),
         b"pcst": (__parse_bool, __render_bool),
+        b"shwm": (__parse_integer, __render_integer, 1),
+        b"stik": (__parse_integer, __render_integer, 1),
+        b"rtng": (__parse_integer, __render_integer, 1),
         b"covr": (__parse_cover, __render_cover),
         b"purl": (__parse_text, __render_text),
         b"egid": (__parse_text, __render_text),
@@ -826,35 +889,43 @@ class MP4Tags(DictProxy, Tags):
 
 
 class MP4Info(StreamInfo):
-    """MPEG-4 stream information.
+    """MP4Info()
+
+    MPEG-4 stream information.
 
     Attributes:
+        bitrate (`int`): bitrate in bits per second, as an int
+        length (`float`): file length in seconds, as a float
+        channels (`int`): number of audio channels
+        sample_rate (`int`): audio sampling rate in Hz
+        bits_per_sample (`int`): bits per sample
+        codec (`mutagen.text`):
+            * if starting with ``"mp4a"`` uses an mp4a audio codec
+              (see the codec parameter in rfc6381 for details e.g.
+              ``"mp4a.40.2"``)
+            * for everything else see a list of possible values at
+              http://www.mp4ra.org/codecs.html
 
-    * bitrate -- bitrate in bits per second, as an int
-    * length -- file length in seconds, as a float
-    * channels -- number of audio channels
-    * sample_rate -- audio sampling rate in Hz
-    * bits_per_sample -- bits per sample
-    * codec (string):
-        * if starting with ``"mp4a"`` uses an mp4a audio codec
-          (see the codec parameter in rfc6381 for details e.g. ``"mp4a.40.2"``)
-        * for everything else see a list of possible values at
-          http://www.mp4ra.org/codecs.html
-
-        e.g. ``"mp4a"``, ``"alac"``, ``"mp4a.40.2"``, ``"ac-3"`` etc.
-    * codec_description (string):
-        Name of the codec used (ALAC, AAC LC, AC-3...). Values might change in
-        the future, use for display purposes only.
+            e.g. ``"mp4a"``, ``"alac"``, ``"mp4a.40.2"``, ``"ac-3"`` etc.
+        codec_description (`mutagen.text`):
+            Name of the codec used (ALAC, AAC LC, AC-3...). Values might
+            change in the future, use for display purposes only.
     """
 
     bitrate = 0
+    length = 0.0
     channels = 0
     sample_rate = 0
     bits_per_sample = 0
     codec = u""
-    codec_name = u""
+    codec_description = u""
 
-    def __init__(self, atoms, fileobj):
+    def __init__(self, *args, **kwargs):
+        if args or kwargs:
+            self.load(*args, **kwargs)
+
+    @convert_error(IOError, MP4StreamInfoError)
+    def load(self, atoms, fileobj):
         try:
             moov = atoms[b"moov"]
         except KeyError:
@@ -868,7 +939,7 @@ class MP4Info(StreamInfo):
             if data[8:12] == b"soun":
                 break
         else:
-            raise MP4StreamInfoError("track has no audio data")
+            raise MP4NoTrackError("track has no audio data")
 
         mdhd = trak[b"mdia", b"mdhd"]
         ok, data = mdhd.read(fileobj)
@@ -956,52 +1027,61 @@ class MP4Info(StreamInfo):
 
 
 class MP4(FileType):
-    """An MPEG-4 audio file, probably containing AAC.
+    """MP4(filething)
+
+    An MPEG-4 audio file, probably containing AAC.
 
     If more than one track is present in the file, the first is used.
     Only audio ('soun') tracks will be read.
 
-    :ivar info: :class:`MP4Info`
-    :ivar tags: :class:`MP4Tags`
+    Arguments:
+        filething (filething)
+
+    Attributes:
+        info (`MP4Info`)
+        tags (`MP4Tags`)
     """
 
     MP4Tags = MP4Tags
 
     _mimes = ["audio/mp4", "audio/x-m4a", "audio/mpeg4", "audio/aac"]
 
-    def load(self, filename):
-        self.filename = filename
-        with open(filename, "rb") as fileobj:
-            try:
-                atoms = Atoms(fileobj)
-            except AtomError as err:
-                reraise(error, err, sys.exc_info()[2])
+    @loadfile()
+    def load(self, filething):
+        fileobj = filething.fileobj
 
+        try:
+            atoms = Atoms(fileobj)
+        except AtomError as err:
+            reraise(error, err, sys.exc_info()[2])
+
+        self.info = MP4Info()
+        try:
+            self.info.load(atoms, fileobj)
+        except MP4NoTrackError:
+            pass
+        except error:
+            raise
+        except Exception as err:
+            reraise(MP4StreamInfoError, err, sys.exc_info()[2])
+
+        if not MP4Tags._can_load(atoms):
+            self.tags = None
+            self._padding = 0
+        else:
             try:
-                self.info = MP4Info(atoms, fileobj)
+                self.tags = self.MP4Tags(atoms, fileobj)
             except error:
                 raise
             except Exception as err:
-                reraise(MP4StreamInfoError, err, sys.exc_info()[2])
-
-            if not MP4Tags._can_load(atoms):
-                self.tags = None
-                self._padding = 0
+                reraise(MP4MetadataError, err, sys.exc_info()[2])
             else:
-                try:
-                    self.tags = self.MP4Tags(atoms, fileobj)
-                except error:
-                    raise
-                except Exception as err:
-                    reraise(MP4MetadataError, err, sys.exc_info()[2])
-                else:
-                    self._padding = self.tags._padding
+                self._padding = self.tags._padding
 
-    def save(self, filename=None, padding=None):
-        super(MP4, self).save(filename, padding=padding)
+    def save(self, *args, **kwargs):
+        """save(filething=None, padding=None)"""
 
-    def delete(self, filename=None):
-        super(MP4, self).delete(filename)
+        super(MP4, self).save(*args, **kwargs)
 
     def add_tags(self):
         if self.tags is None:
@@ -1017,7 +1097,19 @@ class MP4(FileType):
 Open = MP4
 
 
-def delete(filename):
-    """Remove tags from a file."""
+@convert_error(IOError, error)
+@loadfile(method=False, writable=True)
+def delete(filething):
+    """ delete(filething)
 
-    MP4(filename).delete()
+    Arguments:
+        filething (filething)
+    Raises:
+        mutagen.MutagenError
+
+    Remove tags from a file.
+    """
+
+    t = MP4(filething)
+    filething.fileobj.seek(0)
+    t.delete(filething)
