@@ -3,7 +3,7 @@
 from mutagen._util import DictMixin, cdata, insert_bytes, delete_bytes, \
     decode_terminated, dict_match, enum, get_size, BitReader, BitReaderError, \
     resize_bytes, seek_end, mmap_move, verify_fileobj, fileobj_name, \
-    read_full, flags, resize_file, fallback_move
+    read_full, flags, resize_file, fallback_move, encode_endian
 from mutagen._compat import text_type, itervalues, iterkeys, iteritems, PY2, \
     cBytesIO, xrange, BytesIO
 from tests import TestCase, get_temp_empty
@@ -143,6 +143,10 @@ class Tcdata(TestCase):
         self.assertTrue(cdata.char is cdata.int8)
         self.assertTrue(cdata.to_char is cdata.to_int8)
         self.assertTrue(cdata.char_from is cdata.int8_from)
+        assert cdata.int8_max == 2 ** 7 - 1
+        assert cdata.int8_min == - 2 ** 7
+        assert cdata.int8_max == cdata.char_max
+        assert cdata.int8_min == cdata.char_min
 
     def test_char_from_to(self):
         self.assertEqual(cdata.to_char(-2), b"\xfe")
@@ -158,6 +162,10 @@ class Tcdata(TestCase):
         self.assertTrue(cdata.uchar is cdata.uint8)
         self.assertTrue(cdata.to_uchar is cdata.to_uint8)
         self.assertTrue(cdata.uchar_from is cdata.uint8_from)
+        assert cdata.uint8_max == 2 ** 8 - 1
+        assert cdata.uint8_min == 0
+        assert cdata.uint8_max == cdata.uchar_max
+        assert cdata.uint8_min == cdata.uchar_min
 
     def test_short(self):
         self.failUnlessEqual(cdata.short_le(self.ZERO(2)), 0)
@@ -584,6 +592,28 @@ class FileHandlingMockedMMap(FileHandling):
         mmap.mmap = self._orig_mmap
 
 
+class FileHandlingNoMMap(FileHandling):
+    """Disables mmap and makes sure it raises if it still gets used somehow"""
+
+    def setUp(self):
+        from mutagen import _util
+
+        def MockMMap2(*args, **kwargs):
+            assert False
+
+        self._orig_mmap = mmap.mmap
+        mmap.mmap = MockMMap2
+
+        _util.mmap = None
+
+    def tearDown(self):
+        from mutagen import _util
+        import mmap
+
+        _util.mmap = mmap
+        mmap.mmap = self._orig_mmap
+
+
 class Tdict_match(TestCase):
 
     def test_match(self):
@@ -720,6 +750,33 @@ class Tget_size(TestCase):
         self.assertEqual(f.tell(), 1)
 
 
+class Tencode_endian(TestCase):
+
+    def test_other(self):
+        assert encode_endian(u"\xe4", "latin-1") == b"\xe4"
+        assert encode_endian(u"\xe4", "utf-8") == b"\xc3\xa4"
+        with self.assertRaises(LookupError):
+            encode_endian(u"", "nopenope")
+        with self.assertRaises(UnicodeEncodeError):
+            assert encode_endian(u"\u2714", "latin-1")
+        assert encode_endian(u"\u2714", "latin-1", "replace") == b"?"
+
+    def test_utf_16(self):
+        assert encode_endian(u"\xe4", "utf-16", le=True) == b"\xff\xfe\xe4\x00"
+        assert encode_endian(u"\xe4", "utf-16-le") == b"\xe4\x00"
+        assert encode_endian(
+            u"\xe4", "utf-16", le=False) == b"\xfe\xff\x00\xe4"
+        assert encode_endian(u"\xe4", "utf-16-be") == b"\x00\xe4"
+
+    def test_utf_32(self):
+        assert encode_endian(u"\xe4", "utf-32", le=True) == \
+            b"\xff\xfe\x00\x00\xe4\x00\x00\x00"
+        assert encode_endian(u"\xe4", "utf-32-le") == b"\xe4\x00\x00\x00"
+        assert encode_endian(
+            u"\xe4", "utf-32", le=False) == b"\x00\x00\xfe\xff\x00\x00\x00\xe4"
+        assert encode_endian(u"\xe4", "utf-32-be") == b"\x00\x00\x00\xe4"
+
+
 class Tdecode_terminated(TestCase):
 
     def test_all(self):
