@@ -5,11 +5,10 @@
 import os
 import shutil
 
-from tempfile import mkstemp
-
-from tests import TestCase, DATA_DIR
+from tests import TestCase, DATA_DIR, get_temp_copy
 
 import mutagen.apev2
+from mutagen import MutagenError
 from mutagen._compat import PY3, text_type, izip, xrange
 from mutagen.apev2 import APEv2File, APEv2, is_valid_apev2_key, \
     APEBadItemError, error as APEv2Error
@@ -100,7 +99,7 @@ class TAPEWriter(TestCase):
 
     def test_empty(self):
         self.failUnlessRaises(
-            IOError, mutagen.apev2.APEv2,
+            APEv2Error, mutagen.apev2.APEv2,
             os.path.join(DATA_DIR, "emptyfile.mp3"))
 
     def test_tag_at_start(self):
@@ -122,7 +121,7 @@ class TAPEWriter(TestCase):
         filename = SAMPLE + ".tag_at_start"
         tag = mutagen.apev2.APEv2(filename)
         tag.delete()
-        self.failUnlessRaises(IOError, mutagen.apev2.APEv2, filename)
+        self.failUnlessRaises(APEv2Error, mutagen.apev2.APEv2, filename)
         self.failUnlessEqual(
             os.path.getsize(filename), len("tag garbage") * 1000)
 
@@ -171,11 +170,12 @@ class TAPEv2ThenID3v1Writer(TAPEWriter):
 class TAPEv2(TestCase):
 
     def setUp(self):
-        fd, self.filename = mkstemp(".apev2")
-        os.close(fd)
-        shutil.copy(OLD, self.filename)
+        self.filename = get_temp_copy(OLD)
         self.audio = APEv2(self.filename)
 
+    def tearDown(self):
+        os.unlink(self.filename)
+
     def test_invalid_key(self):
         self.failUnlessRaises(
             KeyError, self.audio.__setitem__, u"\u1234", "foo")
@@ -213,10 +213,10 @@ class TAPEv2(TestCase):
         delete(os.path.join(DATA_DIR, "emptyfile.mp3"))
 
     def test_invalid(self):
-        self.failUnlessRaises(IOError, mutagen.apev2.APEv2, "dne")
+        self.failUnlessRaises(MutagenError, mutagen.apev2.APEv2, "dne")
 
     def test_no_tag(self):
-        self.failUnlessRaises(IOError, mutagen.apev2.APEv2,
+        self.failUnlessRaises(MutagenError, mutagen.apev2.APEv2,
                               os.path.join(DATA_DIR, "empty.mp3"))
 
     def test_cases(self):
@@ -269,9 +269,6 @@ class TAPEv2(TestCase):
     def test_pprint(self):
         self.failUnless(self.audio.pprint())
 
-    def tearDown(self):
-        os.unlink(self.filename)
-
 
 class TAPEv2ThenID3v1(TAPEv2):
 
