@@ -8,7 +8,8 @@ from mutagen.apev2 import APEv2
 from mutagen.id3 import ID3, Frames, ID3UnsupportedVersionError, TIT2, \
     CHAP, CTOC, TT1, TCON, COMM, TORY, PIC, MakeID3v1, TRCK, TYER, TDRC, \
     TDAT, TIME, LNK, IPLS, TPE1, BinaryFrame, TIT3, POPM, APIC, CRM, \
-    TALB, TPE2, TSOT, TDEN, TIPL, ParseID3v1, Encoding, ID3Tags, RVAD
+    TALB, TPE2, TSOT, TDEN, TIPL, ParseID3v1, Encoding, ID3Tags, RVAD, \
+    ID3NoHeaderError
 from mutagen.id3._util import BitPaddedInt, error as ID3Error
 from mutagen.id3._tags import determine_bpi, ID3Header, \
     save_frame, ID3SaveConfig
@@ -371,6 +372,22 @@ class TID3Tags(TestCase):
         for key, value in id3.items():
             self.assertEqual(key, value.HashKey)
 
+    def test_text_duplicate_frame_different_encoding(self):
+        id3 = ID3Tags()
+        frame = TPE2(encoding=Encoding.LATIN1, text=[u"foo"])
+        id3._add(frame, False)
+        assert id3.getall("TPE2")[0].encoding == Encoding.LATIN1
+        frame = TPE2(encoding=Encoding.LATIN1, text=[u"bar"])
+        id3._add(frame, False)
+        assert id3.getall("TPE2")[0].encoding == Encoding.LATIN1
+        frame = TPE2(encoding=Encoding.UTF8, text=[u"baz\u0400"])
+        id3._add(frame, False)
+        assert id3.getall("TPE2")[0].encoding == Encoding.UTF8
+
+        frames = id3.getall("TPE2")
+        assert len(frames) == 1
+        assert len(frames[0].text) == 3
+
     def test_add_CRM(self):
         id3 = ID3Tags()
         self.assertRaises(TypeError, id3.add, CRM())
@@ -1052,6 +1069,22 @@ class Issue69_BadV1Year(TestCase):
         self.failUnlessEqual(tag["TDRC"], "1234")
 
 
+class TID3Trailing(TestCase):
+
+    def test_audacious_trailing_id3(self):
+        # https://github.com/quodlibet/mutagen/issues/78
+        # tagged with audacious 3.2.4, both are id3v2 at the end despite the
+        # spec saying it should be before other tags.
+        # Audacious changed it to write in the beginning with 3.4 or 3.5
+        # Now with Audacious 3.7, re-saving the files results in the id3v3
+        # tag moved to the front and the id3v1/apev2 tags left as is at the
+        # end.
+        path = os.path.join(DATA_DIR, 'audacious-trailing-id32-id31.mp3')
+        self.assertRaises(ID3NoHeaderError, ID3, path)
+        path = os.path.join(DATA_DIR, 'audacious-trailing-id32-apev2.mp3')
+        self.assertRaises(ID3NoHeaderError, ID3, path)
+
+
 class TID3Misc(TestCase):
 
     def test_main(self):
