@@ -42,6 +42,7 @@ class TMP3(TestCase):
     silence_mpeg25 = os.path.join(DATA_DIR, 'silence-44-s-mpeg25.mp3')
     lame = os.path.join(DATA_DIR, 'lame.mp3')
     lame_peak = os.path.join(DATA_DIR, 'lame-peak.mp3')
+    lame_broken_short = os.path.join(DATA_DIR, 'lame397v9short.mp3')
 
     def setUp(self):
         self.filename = get_temp_copy(
@@ -54,6 +55,16 @@ class TMP3(TestCase):
         self.mp3_lame = MP3(self.lame)
         self.mp3_lame_peak = MP3(self.lame_peak)
 
+    def test_lame_broken_short(self):
+        # lame <=3.97 wrote broken files
+        f = MP3(self.lame_broken_short)
+        assert f.info.encoder_info == "LAME 3.97.0"
+        assert f.info.encoder_settings == "-V 9"
+        assert f.info.length == 0.0
+        assert f.info.bitrate == 64000
+        assert f.info.bitrate_mode == 2
+        assert f.info.sample_rate == 24000
+
     def test_mode(self):
         from mutagen.mp3 import JOINTSTEREO
         self.failUnlessEqual(self.mp3.info.mode, JOINTSTEREO)
@@ -233,6 +244,18 @@ class TMPEGInfo(TestCase):
         fileobj = cBytesIO(b"")
         self.failUnlessRaises(MP3Error, MPEGInfo, fileobj)
 
+    def test_xing_unknown_framecount(self):
+        frame = (
+            b'\xff\xfb\xe4\x0c\x00\x0f\xf0\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+            b'\x00\x00\x00\x00Info\x00\x00\x00\x02\x00\xb4V@\x00\xb4R\x80\x00'
+            b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
+        )
+        fileobj = cBytesIO(frame)
+        info = MPEGInfo(fileobj)
+        assert info.bitrate == 320000
+        assert info.length > 0
+
 
 class TEasyMP3(TestCase):
 
@@ -330,7 +353,7 @@ class TLAMEHeader(TestCase):
 
         def parse(data):
             data = cBytesIO(data + b"\x00" * (20 - len(data)))
-            return LAMEHeader.parse_version(data)
+            return tuple(LAMEHeader.parse_version(data)[1:])
 
         self.assertEqual(parse(b"LAME3.80"), (u"3.80", False))
         self.assertEqual(parse(b"LAME3.80 "), (u"3.80", False))
@@ -345,6 +368,7 @@ class TLAMEHeader(TestCase):
         self.assertEqual(parse(b"L3.99r"), (u"3.99.1+", True))
         self.assertEqual(parse(b"LAME3100r"), (u"3.100.1+", True))
         self.assertEqual(parse(b"LAME3.90.\x03\xbe\x00"), (u"3.90.0+", True))
+        self.assertEqual(parse(b"LAME3.100"), (u"3.100.0+", True))
 
     def test_invalid(self):
 
@@ -354,15 +378,74 @@ class TLAMEHeader(TestCase):
 
         self.assertRaises(LAMEError, parse, b"")
         self.assertRaises(LAMEError, parse, b"LAME")
-        self.assertRaises(LAMEError, parse, b"LAME3.999")
+        self.assertRaises(LAMEError, parse, b"LAME3.9999")
 
     def test_real(self):
         with open(os.path.join(DATA_DIR, "lame.mp3"), "rb") as h:
             h.seek(36, 0)
             xing = XingHeader(h)
-            self.assertEqual(xing.lame_version, u"3.99.1+")
+            self.assertEqual(xing.lame_version_desc, u"3.99.1+")
             self.assertTrue(xing.lame_header)
             self.assertEqual(xing.lame_header.track_gain_adjustment, 6.0)
+            assert xing.get_encoder_settings() == u"-V 2"
+
+    def test_settings(self):
+        with open(os.path.join(DATA_DIR, "lame.mp3"), "rb") as h:
+            h.seek(36, 0)
+            xing = XingHeader(h)
+        header = xing.lame_header
+
+        def s(major, minor, **kwargs):
+            old = vars(header)
+            for key, value in kwargs.items():
+                assert hasattr(header, key)
+                setattr(header, key, value)
+            r = header.guess_settings(major, minor)
+            header.__dict__.update(old)
+            return r
+
+        assert s(3, 99) == "-V 2"
+        assert s(3, 98) == "-V 2"
+        assert s(3, 97) == "-V 2 --vbr-new"
+        assert s(3, 96) == "-V 2 --vbr-new"
+        assert s(3, 95) == "-V 2 --vbr-new"
+        assert s(3, 94) == "-V 2 --vbr-new"
+        assert s(3, 93) == "-V 2 --vbr-new"
+        assert s(3, 92) == "-V 2 --vbr-new"
+        assert s(3, 91) == "-V 2 --vbr-new"
+        assert s(3, 90) == "-V 2 --vbr-new"
+        assert s(3, 89) == ""
+
+        assert s(3, 91, vbr_method=2) == "--alt-preset 32"
+        assert s(3, 91, vbr_method=2, bitrate=255) == "--alt-preset 255+"
+        assert s(3, 99, vbr_method=2, preset_used=128) == "--preset 128"
+        assert s(3, 99, vbr_method=2, preset_used=0, bitrate=48) == "--abr 48"
+        assert \
+            s(3, 94, vbr_method=2, preset_used=0, bitrate=255) == "--abr 255+"
+        assert s(3, 99, vbr_method=3) == "-V 2 --vbr-old"
+        assert s(3, 94, vbr_method=3) == "-V 2"
+        assert s(3, 99, vbr_method=1, preset_used=1003) == "--preset insane"
+        assert s(3, 93, vbr_method=3, preset_used=1001) == "--preset standard"
+        assert s(3, 93, vbr_method=3, preset_used=1002) == "--preset extreme"
+        assert s(3, 93, vbr_method=3, preset_used=1004) == \
+            "--preset fast standard"
+        assert s(3, 93, vbr_method=3, preset_used=1005) == \
+            "--preset fast extreme"
+        assert s(3, 93, vbr_method=3, preset_used=1006) == "--preset medium"
+        assert s(3, 93, vbr_method=3, preset_used=1007) == \
+            "--preset fast medium"
+        assert s(3, 92, vbr_method=3) == "-V 2"
+        assert s(3, 92, vbr_method=1, preset_used=0, bitrate=254) == "-b 254"
+        assert s(3, 92, vbr_method=1, preset_used=0, bitrate=255) == "-b 255+"
+
+        def skey(major, minor, args):
+            keys = ["vbr_quality", "quality", "vbr_method", "lowpass_filter",
+                    "ath_type"]
+            return s(major, minor, **dict(zip(keys, args)))
+
+        assert skey(3, 91, (1, 2, 4, 19500, 3)) == "--preset r3mix"
+        assert skey(3, 91, (2, 2, 3, 19000, 4)) == "--alt-preset standard"
+        assert skey(3, 91, (2, 2, 3, 19500, 2)) == "--alt-preset extreme"
 
     def test_length(self):
         mp3 = MP3(os.path.join(DATA_DIR, "lame.mp3"))
