@@ -1,14 +1,12 @@
 # -*- coding: utf-8 -*-
 
 import os
-import shutil
-
-from tempfile import mkstemp
 
 from mutagen._compat import cBytesIO
-from mutagen.oggtheora import OggTheora, OggTheoraInfo, delete
+from mutagen.oggtheora import OggTheora, OggTheoraInfo, delete, error
 from mutagen.ogg import OggPage
-from tests import TestCase, DATA_DIR
+
+from tests import TestCase, DATA_DIR, get_temp_copy
 from tests.test_ogg import TOggFileTypeMixin
 
 
@@ -16,10 +14,9 @@ class TOggTheora(TestCase, TOggFileTypeMixin):
     Kind = OggTheora
 
     def setUp(self):
-        original = os.path.join(DATA_DIR, "sample.oggtheora")
-        fd, self.filename = mkstemp(suffix='.ogg')
-        os.close(fd)
-        shutil.copy(original, self.filename)
+        self.filename = get_temp_copy(
+            os.path.join(DATA_DIR, "sample.oggtheora"))
+
         self.audio = OggTheora(self.filename)
         self.audio2 = OggTheora(
             os.path.join(DATA_DIR, "sample_length.oggtheora"))
@@ -30,18 +27,20 @@ class TOggTheora(TestCase, TOggFileTypeMixin):
         os.unlink(self.filename)
 
     def test_theora_bad_version(self):
-        page = OggPage(open(self.filename, "rb"))
+        with open(self.filename, "rb") as h:
+            page = OggPage(h)
         packet = page.packets[0]
         packet = packet[:7] + b"\x03\x00" + packet[9:]
         page.packets = [packet]
         fileobj = cBytesIO(page.write())
-        self.failUnlessRaises(IOError, OggTheoraInfo, fileobj)
+        self.failUnlessRaises(error, OggTheoraInfo, fileobj)
 
     def test_theora_not_first_page(self):
-        page = OggPage(open(self.filename, "rb"))
+        with open(self.filename, "rb") as h:
+            page = OggPage(h)
         page.first = False
         fileobj = cBytesIO(page.write())
-        self.failUnlessRaises(IOError, OggTheoraInfo, fileobj)
+        self.failUnlessRaises(error, OggTheoraInfo, fileobj)
 
     def test_vendor(self):
         self.failUnless(
@@ -50,9 +49,9 @@ class TOggTheora(TestCase, TOggFileTypeMixin):
 
     def test_not_my_ogg(self):
         fn = os.path.join(DATA_DIR, 'empty.ogg')
-        self.failUnlessRaises(IOError, type(self.audio), fn)
-        self.failUnlessRaises(IOError, self.audio.save, fn)
-        self.failUnlessRaises(IOError, self.audio.delete, fn)
+        self.failUnlessRaises(error, type(self.audio), fn)
+        self.failUnlessRaises(error, self.audio.save, fn)
+        self.failUnlessRaises(error, self.audio.delete, fn)
 
     def test_length(self):
         self.failUnlessAlmostEqual(5.5, self.audio.info.length, 1)
